@@ -67,14 +67,14 @@ dbus_create_service_instance (const char *service_name)
 
 	proxy = dbus_g_proxy_new_for_name (connection,
 					   "org.freedesktop.DBus",
-					   "/org/freedesktop/Dbus",
+					   "/org/freedesktop/DBus",
 					   "org.freedesktop.DBus");
 	error = NULL;
 	if (!dbus_g_proxy_call (proxy,
 				"RequestName",
 				&error,
 				G_TYPE_STRING, service_name,
-				G_TYPE_UINT, 0,
+				G_TYPE_UINT, DBUS_NAME_FLAG_ALLOW_REPLACEMENT,
 				G_TYPE_INVALID,
 				G_TYPE_UINT, &request_name_result,
 				G_TYPE_INVALID))
@@ -83,9 +83,12 @@ dbus_create_service_instance (const char *service_name)
 		           "Got error \"%s\"\n",
 		           error->message);
 		g_error_free (error);
+		g_object_unref (proxy);
 		return NULL;
 	}
 
+	g_object_unref (proxy);
+
 	if (request_name_result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
 	{
 		g_warning ("Another instance has already registered %s", service_name);
