@@ -575,6 +575,7 @@ stack_notify_handler (Stack*                 self,
 	GdkPixbuf* pixbuf     = NULL;
 	gboolean   new_bubble = FALSE;
 	gboolean   turn_into_dialog;
+	guint      real_id;
 
 	// check max. allowed limit queue-size
 	if (g_list_length (self->list) > MAX_STACK_SIZE)
@@ -660,6 +661,8 @@ stack_notify_handler (Stack*                 self,
 		}
 	}
 
+	real_id = bubble_get_id (bubble);
+
 	if (hints)
 	{
 		data   = (GValue*) g_hash_table_lookup (hints, "x-canonical-private-synchronous");
@@ -671,6 +674,12 @@ stack_notify_handler (Stack*                 self,
 			{
 				g_object_unref (bubble);
 				bubble = sync_bubble;
+
+				bubble_set_title (sync_bubble, summary ? summary : "");
+				bubble_set_message_body (sync_bubble, body ? body : "");
+				bubble_set_value (sync_bubble, -2);
+
+				bubble_determine_layout (sync_bubble);
 			}
 
 			if (data && G_VALUE_HOLDS_STRING (data))
@@ -750,7 +759,7 @@ stack_notify_handler (Stack*                 self,
 	{
 		stack_display_sync_bubble (self, bubble);
 	} else {
-		stack_push_bubble (self, bubble);
+		real_id = stack_push_bubble (self, bubble);
 
 		if (! new_bubble && bubble_is_append_allowed (bubble))
 			log_bubble (bubble, app_name, "appended");
@@ -769,11 +778,11 @@ stack_notify_handler (Stack*                 self,
 
 		/* update the layout of the stack;
 		 * this will also open the new bubble */
-		stack_layout (self);
+		if (!stack_layout (self))
+			bubble = NULL;
 	}
 
-	if (bubble)
-		dbus_g_method_return (context, bubble_get_id (bubble));
+	dbus_g_method_return (context, real_id);
 
 	// FIXME: this is a temporary work-around, I do not like at all, until
 	// the heavy memory leakage of notify-osd is fully fixed...
@@ -885,6 +894,9 @@ stack_get_slot_position (Stack* self,
                          gint*  x,
                          gint*  y)
 {
+	GdkScreen* screen        = NULL;
+	gboolean   is_composited = FALSE;
+
 	// sanity checks
 	if (!x && !y)
 		return;
@@ -904,7 +916,9 @@ stack_get_slot_position (Stack* self,
 	}
 
 	// initialize x and y
-	defaults_get_top_corner (self->defaults, x, y);
+	defaults_get_top_corner (self->defaults, &screen, x, y);
+
+	is_composited = gdk_screen_is_composited (screen);
 
 	// differentiate returned top-left corner for top and bottom slot
 	// depending on the placement 
@@ -920,12 +934,12 @@ stack_get_slot_position (Stack* self,
 				*y += defaults_get_desktop_height (d) / 2 -
 				      EM2PIXELS (defaults_get_bubble_vert_gap (d) / 2.0f, d) -
 				      bubble_height +
-				      EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+				      EM2PIXELS (defaults_get_bubble_shadow_size (d, is_composited), d);
 			// the position for the async. bubble
 			else if (slot == SLOT_BOTTOM)
 				*y += defaults_get_desktop_height (d) / 2 +
 				      EM2PIXELS (defaults_get_bubble_vert_gap (d) / 2.0f, d) -
-				      EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+				      EM2PIXELS (defaults_get_bubble_shadow_size (d, is_composited), d);
 		break;
 
 		case GRAVITY_NORTH_EAST:
@@ -945,14 +959,14 @@ stack_get_slot_position (Stack* self,
 						*y += EM2PIXELS (defaults_get_icon_size (d), d) +
 						      2 * EM2PIXELS (defaults_get_margin_size (d), d) +
 						      EM2PIXELS (defaults_get_bubble_vert_gap (d), d); /* +
-						      2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);*/
+						      2 * EM2PIXELS (defaults_get_bubble_shadow_size (d, is_composited), d);*/
 					break;
 
 					case SLOT_ALLOCATION_DYNAMIC:
 						g_assert (stack_is_slot_vacant (self, SLOT_TOP) == OCCUPIED);
 						*y += bubble_get_height (self->slots[SLOT_TOP]) +
 						      EM2PIXELS (defaults_get_bubble_vert_gap (d), d) -
-						      2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+						      2 * EM2PIXELS (defaults_get_bubble_shadow_size (d, is_composited), d);
 					break;
 
 					default:
