@@ -328,7 +328,7 @@ expose_handler (GtkWidget*      window,
 					 a.height,
 					 g_distance,
 					 1.0f - g_distance);
-		gtk_window_set_opacity (GTK_WINDOW (window), 0.3f + g_distance * 0.7f);
+		gtk_widget_set_opacity (window, 0.3f + g_distance * 0.7f);
 	}
 	else
 	{
@@ -340,7 +340,7 @@ expose_handler (GtkWidget*      window,
 					 a.height,
 					 g_distance,
 					 0.0f);
-		gtk_window_set_opacity (GTK_WINDOW (window), 1.0f);
+		gtk_widget_set_opacity (window, 1.0f);
 	}
 
 	return TRUE;
@@ -482,10 +482,19 @@ pointer_update (GtkWidget* window)
 
 	if (gtk_widget_get_realized (window))
 	{
-		gint distance_x = 0;
-		gint distance_y = 0;
+		GdkDeviceManager *device_manager;
+		GdkDevice        *device;
+		gint              distance_x;
+		gint              distance_y;
+
+		device_manager = gdk_display_get_device_manager (gtk_widget_get_display (window));
+		device = gdk_device_manager_get_client_pointer (device_manager);
+		gdk_window_get_device_position (gtk_widget_get_window (window),
+		                                device,
+		                                &pointer_rel_x,
+		                                &pointer_rel_y,
+		                                NULL);
 
-		gtk_widget_get_pointer (window, &pointer_rel_x, &pointer_rel_y);
 		gtk_window_get_position (GTK_WINDOW (window), &win_x, &win_y);
 		pointer_abs_x = win_x + pointer_rel_x;
 		pointer_abs_y = win_y + pointer_rel_y;
@@ -714,7 +723,7 @@ main (int    argc,
 	gtk_window_set_keep_above (GTK_WINDOW (window), TRUE);
 	gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
 	gtk_window_set_accept_focus (GTK_WINDOW (window), FALSE);
-	gtk_window_set_opacity (GTK_WINDOW (window), 1.0f);
+	gtk_widget_set_opacity (window, 1.0f);
 	gtk_widget_set_size_request (window,
 				     (gint) (BUBBLE_WIDTH + 2.0f *
 					     BUBBLE_SHADOW_SIZE),
